@@ -1,23 +1,27 @@
-/* Clearlooks Gummy style
- * Copyright (C) 2007 Andrea Cimitan
+/* Clearlooks - a cairo based GTK+ engine
+ * Copyright (C) 2007-2008 Andrea Cimitan <andrea.cimitan@gmail.com>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Project contact: <gnome-themes-list@gnome.org>
+ *
+ *
+ * This file defines the Clearlooks Gummy style
  *
- * Written by Andrea Cimitan <andrea.cimitan@gmail.com>
  */
+ 
 
 #include "clearlooks_draw.h"
 #include "clearlooks_style.h"
@@ -33,10 +37,6 @@
 #define SHADE_CENTER_TOP 1.02
 #define SHADE_BOTTOM 0.94
 
-/* Topleft highlight */
-#define TOPLEFT_HIGHLIGHT_SHADE 1.3
-#define TOPLEFT_HIGHLIGHT_ALPHA 0.4
-
 /* Listview */
 #define LISTVIEW_SHADE_TOP 1.06
 #define LISTVIEW_SHADE_CENTER_TOP 1.02
@@ -75,18 +75,6 @@ clearlooks_draw_gummy_gradient (cairo_t          *cr,
 	cairo_pattern_destroy (pt);
 }
 
-static void
-clearlooks_set_mixed_color (cairo_t          *cr,
-                            const CairoColor *color1,
-                            const CairoColor *color2,
-                            gdouble mix_factor)
-{
-	CairoColor composite;
-
-	ge_mix_color (color1, color2, mix_factor, &composite);
-	ge_cairo_set_color (cr, &composite);
-}
-
 static void
 clearlooks_gummy_draw_highlight_and_shade (cairo_t                *cr,
                                            const CairoColor       *bg_color,
@@ -141,29 +129,6 @@ clearlooks_gummy_draw_highlight_and_shade (cairo_t                *cr,
 	cairo_restore (cr);
 }
 
-static void
-clearlooks_gummy_draw_top_left_highlight (cairo_t                *cr,
-                                          const CairoColor       *color,
-                                          const WidgetParameters *params,
-                                          int width, int height, gdouble radius)
-{
-	CairoColor hilight;
-
-	double light_top = params->ythickness - 1,
-	       light_bottom = height - params->ythickness - 1,
-	       light_left = params->xthickness - 1,
-	       light_right = width - params->xthickness - 1;
-
-	ge_shade_color (color, TOPLEFT_HIGHLIGHT_SHADE, &hilight);
-	cairo_move_to (cr, light_left, light_bottom - (int)radius/2);
-
-	ge_cairo_rounded_corner (cr, light_left, light_top, radius, params->corners & CR_CORNER_TOPLEFT);
-
-	cairo_line_to (cr, light_right - (int)radius/2, light_top);
-	cairo_set_source_rgba (cr, hilight.r, hilight.g, hilight.b, TOPLEFT_HIGHLIGHT_ALPHA);
-	cairo_stroke (cr);
-}
-
 static void
 clearlooks_gummy_draw_button (cairo_t                *cr,
                               const ClearlooksColors *colors,
@@ -182,48 +147,46 @@ clearlooks_gummy_draw_button (cairo_t                *cr,
 	cairo_translate (cr, x, y);
 	cairo_set_line_width (cr, 1.0);
 
-	if (params->xthickness == 3)
+	if (params->xthickness >= 3)
+	{
 		xoffset = 1;
-	if (params->ythickness == 3)
 		yoffset = 1;
+	}
 
 	radius = MIN (params->radius, MIN ((width - 2.0 - 2*xoffset) / 2.0, (height - 2.0 - 2*yoffset) / 2.0));
 
-	if (params->xthickness == 3 || params->ythickness == 3)
+	if (params->xthickness >= 3 || params->ythickness >= 3)
 	{
-		cairo_translate (cr, 0.5, 0.5);
-
 		if (params->enable_shadow && !params->active && !params->disabled && !params->is_default)
 		{
 			CairoColor shadow;
 
 			radius = MIN (params->radius, MIN ((width - 2.0 - 2*xoffset) / 2.0 - 1.0, (height - 2.0 - 2*yoffset) / 2.0 - 1.0));
 
-			ge_cairo_rounded_rectangle (cr, 0, 0, width-1, height-1, radius+1, params->corners);
+			ge_cairo_inner_rounded_rectangle (cr, 0, 0, width, height, radius+1, params->corners);
 			ge_shade_color (&params->parentbg, 0.97, &shadow);
 			ge_cairo_set_color (cr, &shadow);
 			cairo_stroke (cr);
 
-			ge_cairo_rounded_rectangle (cr, 1, 1, width-2, height-2, radius+1, params->corners);
+			ge_cairo_inner_rounded_rectangle (cr, 1, 1, width-1, height-1, radius+1, params->corners);
 			ge_shade_color (&params->parentbg, 0.93, &shadow);
 			ge_cairo_set_color (cr, &shadow);
 			cairo_stroke (cr);
 		}
 
-		if (params->is_default)
+		if (params->is_default && !params->disabled)
 		{
 			CairoColor shadow = colors->spot[1];
 
 			radius = MIN (params->radius, MIN ((width - 2.0 - 2*xoffset) / 2.0 - 1.0, (height - 2.0 - 2*yoffset) / 2.0 - 1.0));
 
-			ge_cairo_rounded_rectangle (cr, 0, 0, width-1, height-1, radius+1, params->corners);
+			ge_cairo_inner_rounded_rectangle (cr, 0, 0, width, height, radius+1, params->corners);
 			clearlooks_set_mixed_color (cr, &params->parentbg, &shadow, 0.5);
 			cairo_stroke (cr);
 		}
 
 		if (!(params->enable_shadow && !params->active && !params->disabled))
-			params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, width-1, height-1, params->radius+1, params->corners);
-		cairo_translate (cr, -0.5, -0.5);
+			params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, width, height, params->radius+1, params->corners);
 	}
 
 	clearlooks_draw_gummy_gradient (cr, xoffset+1, yoffset+1,
@@ -276,8 +239,9 @@ clearlooks_gummy_draw_button (cairo_t                *cr,
 
 	if (!params->active)
 	{
-		cairo_translate (cr, 0.5, 0.5);
-		clearlooks_gummy_draw_top_left_highlight (cr, &fill, params, width, height, radius);
+		params->style_functions->draw_top_left_highlight (cr, &fill, params, 1+xoffset, 1+xoffset,
+		                                                  width-(1+xoffset)*2, height-(1+xoffset)*2,
+		                                                  radius, params->corners);
 	}
 	cairo_restore (cr);
 }
@@ -286,36 +250,52 @@ static void
 clearlooks_gummy_draw_entry (cairo_t                *cr,
                              const ClearlooksColors *colors,
                              const WidgetParameters *params,
+                             const FocusParameters  *focus,
                              int x, int y, int width, int height)
 {
 	const CairoColor *base = &colors->base[params->state_type];
 	CairoColor border = colors->shade[params->disabled ? 4 : 6];
 	double radius = MIN (params->radius, MIN ((width - 4.0) / 2.0, (height - 4.0) / 2.0));
+	int xoffset, yoffset;
 
 	if (params->focus)
-		border = colors->spot[2];
+		border = focus->color;
 
-	cairo_translate (cr, x+0.5, y+0.5);
+	cairo_save (cr);
+
+	cairo_translate (cr, x, y);
 	cairo_set_line_width (cr, 1.0);
 
-	/* Fill the background to get the correct corners. */
-	cairo_rectangle (cr, -0.5, -0.5, width, height);
-	ge_cairo_set_color (cr, &params->parentbg);
-	cairo_fill (cr);
+	if (params->xthickness >= 3 && params->ythickness >= 3)
+	{
+		params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, width, height, radius+1, params->corners);
+		xoffset = 1;
+		yoffset = 1;
+	}
+	else
+	{
+		xoffset = 0;
+		yoffset = 0;
+	}
 
-	/* Fill with the base color, because it was just cleared above */
-	cairo_rectangle (cr, 1.5, 1.5, width-4, height-4);
+	/* Now fill the area we want to be base[NORMAL]. */
+	ge_cairo_rounded_rectangle (cr, xoffset + 1, yoffset + 1,
+	                            width-2*(xoffset + 1), height-2*(yoffset + 1),
+	                            MAX(0, radius-1), params->corners);
 	ge_cairo_set_color (cr, base);
 	cairo_fill (cr);
 
-	params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, width-1, height-1, radius+1, params->corners);
-
 	/* Draw the inner shadow */
 	if (params->focus)
 	{
-		/* ge_cairo_rounded_rectangle (cr, 2, 2, width-5, height-5, RADIUS-1, params->corners); */
-		clearlooks_set_mixed_color (cr, base, &colors->spot[1], 0.5);
-		ge_cairo_stroke_rectangle (cr, 2, 2, width-5, height-5);
+		CairoColor focus_shadow;
+		ge_shade_color (&border, 1.61, &focus_shadow);
+		
+		clearlooks_set_mixed_color (cr, base, &focus_shadow, 0.5);
+		ge_cairo_inner_rounded_rectangle (cr, xoffset + 1, yoffset + 1,
+		                                  width-2*(xoffset + 1), height-2*(yoffset + 1),
+		                                  MAX(0, radius-1), params->corners);
+		cairo_stroke (cr);
 	}
 	else
 	{
@@ -323,20 +303,20 @@ clearlooks_gummy_draw_entry (cairo_t                *cr,
 		ge_shade_color (&border, 0.92, &shadow);
 
 		cairo_set_source_rgba (cr, shadow.r, shadow.g, shadow.b, params->disabled ? 0.09 : 0.18);
-		/*
-		cairo_move_to (cr, 2, height-3);
-		cairo_arc (cr, params->xthickness+RADIUS-1, params->ythickness+RADIUS-1, RADIUS, G_PI, 270*(G_PI/180));
-		cairo_line_to (cr, width-3, 2);
-		*/
-		cairo_move_to (cr, 2, height-3);
-		cairo_line_to (cr, 2, 2);
-		cairo_line_to (cr, width-3, 2);
+
+		cairo_set_line_cap (cr, CAIRO_LINE_CAP_BUTT);
+		cairo_move_to (cr, xoffset + 1.5, height-radius);
+		cairo_arc (cr, xoffset + 1.5 + MAX(0, radius-1), yoffset + 1.5 + MAX(0, radius-1),
+		           MAX(0, radius-1), G_PI, 270*(G_PI/180));
+		cairo_line_to (cr, width-radius, yoffset + 1.5);
 		cairo_stroke (cr);
 	}
 
-	ge_cairo_rounded_rectangle (cr, 1, 1, width-3, height-3, radius, params->corners);
+	ge_cairo_inner_rounded_rectangle (cr, xoffset, yoffset, width-2*xoffset, height-2*yoffset, radius, params->corners);
 	ge_cairo_set_color (cr, &border);
 	cairo_stroke (cr);
+
+	cairo_restore (cr);
 }
 
 static void
@@ -354,12 +334,6 @@ clearlooks_gummy_draw_progressbar_trough (cairo_t                *cr,
 
 	cairo_set_line_width (cr, 1.0);
 
-	/* Fill with bg color */
-	ge_cairo_set_color (cr, &colors->bg[params->state_type]);
-
-	cairo_rectangle (cr, x, y, width, height);
-	cairo_fill (cr);
-
 	/* Create trough box */
 	ge_cairo_rounded_rectangle (cr, x+1, y+1, width-2, height-2, radius, params->corners);
 	ge_cairo_set_color (cr, &colors->shade[2]);
@@ -455,7 +429,7 @@ clearlooks_gummy_draw_progressbar_fill (cairo_t                     *cr,
 	cairo_pattern_destroy (pattern);
 
 	/* Draw the strokes */
-	while (tile_pos <= width+x_step)
+	while (stroke_width > 0 && tile_pos <= width+x_step)
 	{
 		cairo_move_to (cr, stroke_width/2-x_step, 0);
 		cairo_line_to (cr, stroke_width-x_step,   0);
@@ -568,20 +542,21 @@ clearlooks_gummy_scale_draw_gradient (cairo_t          *cr,
 	ge_shade_color (fill, in? 0.95 : 1.1, &f1);
 	ge_shade_color (fill, in? 1.05 : 0.9, &f2);
 
-	pattern = cairo_pattern_create_linear (0, 0, horizontal ? 0 :  width, horizontal ? height : 0);
+	pattern = cairo_pattern_create_linear (0.5, 0.5, horizontal ? 0.5 :  width + 1.0, horizontal ? height + 1.0 : 0.5);
 	cairo_pattern_add_color_stop_rgba (pattern, 0.0, f1.r, f1.g, f1.b, f1.a);
 	cairo_pattern_add_color_stop_rgba (pattern, 1.0, f2.r, f2.g, f2.b, f2.a);
 
-	cairo_rectangle (cr, x+0.5, y+0.5, width-1, height-1);
+	cairo_rectangle (cr, x, y, width, height);
 	cairo_set_source (cr, pattern);
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
 
 	clearlooks_set_mixed_color (cr, border, fill, 0.2);
-	ge_cairo_stroke_rectangle (cr, x, y, width, height);
+	ge_cairo_inner_rectangle (cr, x, y, width, height);
+	cairo_stroke (cr);
 }
 
-#define TROUGH_SIZE 6
+#define TROUGH_SIZE 7
 static void
 clearlooks_gummy_draw_scale_trough (cairo_t                *cr,
                                     const ClearlooksColors *colors,
@@ -591,66 +566,64 @@ clearlooks_gummy_draw_scale_trough (cairo_t                *cr,
 {
 	int    trough_width, trough_height;
 	double translate_x, translate_y;
+	CairoColor fill, border;
+	gboolean in;
+
+	cairo_save (cr);
 
 	if (slider->horizontal)
 	{
-		trough_width  = width-3;
-		trough_height = TROUGH_SIZE-2;
+		trough_width  = width;
+		trough_height = TROUGH_SIZE;
 
-		translate_x   = x + 0.5;
-		translate_y   = y + 0.5 + (height/2) - (TROUGH_SIZE/2);
+		translate_x   = x;
+		translate_y   = y + (height/2) - (TROUGH_SIZE/2);
 	}
 	else
 	{
-		trough_width  = TROUGH_SIZE-2;
-		trough_height = height-3;
+		trough_width  = TROUGH_SIZE;
+		trough_height = height;
 
-		translate_x   = x + 0.5 + (width/2) - (TROUGH_SIZE/2);
-		translate_y  = y + 0.5;
+		translate_x   = x + (width/2) - (TROUGH_SIZE/2);
+		translate_y   = y;
 	}
 
 	cairo_set_line_width (cr, 1.0);
 	cairo_translate (cr, translate_x, translate_y);
 
 	if (!slider->fill_level)
-		params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, trough_width+2, trough_height+2, 0, 0);
-
-	cairo_translate (cr, 1, 1);
+		params->style_functions->draw_inset (cr, &params->parentbg, 0, 0, trough_width, trough_height, 0, 0);
 
 	if (!slider->lower && !slider->fill_level)
 	{
-		CairoColor fill;
-
-		/* 0.896 = shade[2], as previous code, but now is shading parentbg */
 		ge_shade_color (&params->parentbg, 0.896, &fill);
-
-		clearlooks_gummy_scale_draw_gradient (cr,
-		                                      &fill, /* bottom */
-		                                      &colors->shade[6], /* border */
-		                                      0, 0, trough_width, trough_height,
-		                                      slider->horizontal, TRUE);
+		border = colors->shade[6];
+		in = TRUE;
 	}
 	else if (!slider->fill_level)
-		clearlooks_gummy_scale_draw_gradient (cr,
-		                                      &colors->spot[1], /* bottom */
-		                                      &colors->spot[2], /* border */
-		                                      0, 0, trough_width, trough_height,
-		                                      slider->horizontal, FALSE);
+	{
+		fill = colors->spot[1];
+		border = colors->spot[2];
+		in = FALSE;
+	}
 	else
 	{
-		CairoColor c1 = colors->spot[1];
-		CairoColor c2 = colors->spot[2];
+		fill = colors->spot[1];
+		border = colors->spot[2];
 
-		c1.a = 0.25;
-		c2.a = 0.25;
-
-		clearlooks_gummy_scale_draw_gradient (cr,
-		                                      &c1, /* bottom */
-		                                      &c2, /* border */
-		                                      0, 0, trough_width, trough_height,
-		                                      slider->horizontal, FALSE);
+		fill.a = 0.25;
+		border.a = 0.25;
+		
+		in = FALSE;
 	}
 
+	clearlooks_gummy_scale_draw_gradient (cr,
+	                                      &fill,
+	                                      &border,
+	                                      1, 1, trough_width - 2, trough_height - 2,
+	                                      slider->horizontal, in);
+
+	cairo_restore (cr);
 }
 
 static void
@@ -942,8 +915,6 @@ clearlooks_gummy_draw_slider (cairo_t                *cr,
 	cairo_set_line_width (cr, 1.0);
 	cairo_translate      (cr, x, y);
 
-	cairo_translate (cr, -0.5, -0.5);
-
 	ge_shade_color (&colors->bg[params->state_type], 1.0, &fill);
 	if (params->prelight)
 		ge_shade_color (&fill, 1.04, &fill);
@@ -981,7 +952,7 @@ clearlooks_gummy_draw_slider (cairo_t                *cr,
 	}
 	cairo_stroke (cr);
 
-	clearlooks_gummy_draw_top_left_highlight (cr, &fill, params, width, height, 2.0);
+	params->style_functions->draw_top_left_highlight (cr, &fill, params, 1, 1, width-2, height-2, 2.0, params->corners);
 }
 
 static void
@@ -998,9 +969,9 @@ clearlooks_gummy_draw_slider_button (cairo_t                *cr,
 	if (!slider->horizontal)
 		ge_cairo_exchange_axis (cr, &x, &y, &width, &height);
 
-	cairo_translate (cr, x+0.5, y+0.5);
+	cairo_translate (cr, x, y);
 
-	params->style_functions->draw_shadow (cr, colors, radius, width-1, height-1);
+	params->style_functions->draw_shadow (cr, colors, radius, width, height);
 	params->style_functions->draw_slider (cr, colors, params, 1, 1, width-2, height-2);
 }
 
@@ -1017,7 +988,6 @@ clearlooks_gummy_draw_scrollbar_stepper (cairo_t                          *cr,
 	CairoColor fill;
 	CairoColor shade1, shade2, shade3;
 	cairo_pattern_t *pattern;
-	ShadowParameters shadow;
 	double radius = MIN (widget->radius, MIN ((width - 2.0) / 2.0, (height - 2.0) / 2.0));
 
 	if (scrollbar->horizontal)
@@ -1026,6 +996,16 @@ clearlooks_gummy_draw_scrollbar_stepper (cairo_t                          *cr,
 			corners = CR_CORNER_TOPLEFT | CR_CORNER_BOTTOMLEFT;
 		else if (stepper->stepper == CL_STEPPER_D)
 			corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
+
+		if (stepper->stepper == CL_STEPPER_B)
+		{
+			x -= 1;
+			width += 1;
+		}
+		else if (stepper->stepper == CL_STEPPER_C)
+		{
+			width += 1;
+		}
 	}
 	else
 	{
@@ -1033,6 +1013,16 @@ clearlooks_gummy_draw_scrollbar_stepper (cairo_t                          *cr,
 			corners = CR_CORNER_TOPLEFT | CR_CORNER_TOPRIGHT;
 		else if (stepper->stepper == CL_STEPPER_D)
 			corners = CR_CORNER_BOTTOMLEFT | CR_CORNER_BOTTOMRIGHT;
+
+		if (stepper->stepper == CL_STEPPER_B)
+		{
+			y -= 1;
+			height += 1;
+		}
+		else if (stepper->stepper == CL_STEPPER_C)
+		{
+			height += 1;
+		}
 	}
 
 	cairo_translate (cr, x, y);
@@ -1058,18 +1048,12 @@ clearlooks_gummy_draw_scrollbar_stepper (cairo_t                          *cr,
 	cairo_fill (cr);
 	cairo_pattern_destroy (pattern);
 
-	cairo_translate (cr, 0.5, 0.5);
-	clearlooks_gummy_draw_top_left_highlight (cr, &fill, widget, width, height,
-	                                          (stepper->stepper == CL_STEPPER_A) ? radius : 0);
-	cairo_translate (cr, -0.5, -0.5);
+	widget->style_functions->draw_top_left_highlight (cr, &fill, widget, 1, 1, width - 2, height - 2,
+	                                                  radius, corners);
 
 	ge_cairo_rounded_rectangle (cr, 0.5, 0.5, width-1, height-1, radius, corners);
 	clearlooks_set_mixed_color (cr, border, &fill, 0.2);
 	cairo_stroke (cr);
-
-	cairo_translate (cr, 0.5, 0.5);
-	shadow.shadow  = CL_SHADOW_OUT;
-	shadow.corners = corners;
 }
 
 static void
@@ -1137,7 +1121,7 @@ clearlooks_gummy_draw_scrollbar_slider (cairo_t                   *cr,
 
 	cairo_set_line_width (cr, 1);
 
-	ge_shade_color (&fill, TOPLEFT_HIGHLIGHT_SHADE, &hilight);
+	ge_shade_color (&fill, widget->style_constants->topleft_highlight_shade, &hilight);
 	ge_shade_color (&fill, SHADE_TOP, &shade1);
 	ge_shade_color (&fill, SHADE_CENTER_TOP, &shade2);
 	ge_shade_color (&fill, SHADE_BOTTOM, &shade3);
@@ -1162,7 +1146,7 @@ clearlooks_gummy_draw_scrollbar_slider (cairo_t                   *cr,
 		cairo_move_to (cr, 1.5, height-1.5);
 		cairo_line_to (cr, 1.5, 1.5);
 		cairo_line_to (cr, width-1.5, 1.5);
-		cairo_set_source_rgba (cr, hilight.r, hilight.g, hilight.b, TOPLEFT_HIGHLIGHT_ALPHA);
+		cairo_set_source_rgba (cr, hilight.r, hilight.g, hilight.b, widget->style_constants->topleft_highlight_alpha);
 		cairo_stroke (cr);
 	}
 
@@ -1430,6 +1414,13 @@ clearlooks_gummy_draw_radiobutton (cairo_t                  *cr,
 	cairo_pattern_t *pt;
 	gboolean inconsistent;
 	gboolean draw_bullet = (checkbox->shadow_type == GTK_SHADOW_IN);
+	gdouble w, h, cx, cy, radius;
+
+	w = (gdouble) width;
+	h = (gdouble) height;
+	cx = width / 2.0;
+	cy = height / 2.0;
+	radius = MIN (width, height) / 2.0;
 
 	inconsistent = (checkbox->shadow_type == GTK_SHADOW_ETCHED_IN);
 	draw_bullet |= inconsistent;
@@ -1451,7 +1442,7 @@ clearlooks_gummy_draw_radiobutton (cairo_t                  *cr,
 	ge_shade_color (&widget->parentbg, 0.9, &shadow);
 	ge_shade_color (&widget->parentbg, 1.1, &highlight);
 
-	pt = cairo_pattern_create_linear (0, 0, 13, 13);
+	pt = cairo_pattern_create_linear (0, 0, radius * 2.0, radius * 2.0);
 	cairo_pattern_add_color_stop_rgb (pt, 0.0, shadow.r, shadow.b, shadow.g);
 	cairo_pattern_add_color_stop_rgba (pt, 0.5, shadow.r, shadow.b, shadow.g, 0.5);
 	cairo_pattern_add_color_stop_rgba (pt, 0.5, highlight.r, highlight.g, highlight.b, 0.5);
@@ -1459,15 +1450,15 @@ clearlooks_gummy_draw_radiobutton (cairo_t                  *cr,
 
 	cairo_translate (cr, x, y);
 
-	cairo_set_line_width (cr, 2);
-	cairo_arc (cr, 7, 7, 6, 0, G_PI*2);
+	cairo_set_line_width (cr, MAX (1.0, floor (radius/3)));
+	cairo_arc (cr, ceil (cx), ceil (cy), floor (radius - 0.1), 0, G_PI*2);
 	cairo_set_source (cr, pt);
 	cairo_stroke (cr);
 	cairo_pattern_destroy (pt);
 
-	cairo_set_line_width (cr, 1);
+	cairo_set_line_width (cr, MAX (1.0, floor (radius/6)));
 
-	cairo_arc (cr, 7, 7, 5.5, 0, G_PI*2);
+	cairo_arc (cr, ceil (cx), ceil (cy), MAX (1.0, ceil (radius) - 1.5), 0, G_PI*2);
 
 	if (!widget->disabled)
 	{
@@ -1485,22 +1476,24 @@ clearlooks_gummy_draw_radiobutton (cairo_t                  *cr,
 	{
 		if (inconsistent)
 		{
-			cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
-			cairo_set_line_width (cr, 4);
+			gdouble line_width = floor (radius * 2 / 3);
 
-			cairo_move_to (cr, 5, 7);
-			cairo_line_to (cr, 9, 7);
+			cairo_set_line_cap(cr, CAIRO_LINE_CAP_ROUND);
+			cairo_set_line_width (cr, line_width);
+
+			cairo_move_to (cr, ceil (cx - radius/3.0 - line_width) + line_width, ceil (cy - line_width) + line_width);
+			cairo_line_to (cr, floor (cx + radius/3.0 + line_width) - line_width, ceil (cy - line_width) + line_width);
 
 			ge_cairo_set_color (cr, dot);
 			cairo_stroke (cr);
 		}
 		else
 		{
-			cairo_arc (cr, 7, 7, 3, 0, G_PI*2);
+			cairo_arc (cr, ceil (cx), ceil (cy), floor (radius/2.0), 0, G_PI*2);
 			ge_cairo_set_color (cr, dot);
 			cairo_fill (cr);
 
-			cairo_arc (cr, 6, 6, 1, 0, G_PI*2);
+			cairo_arc (cr, floor (cx - radius/10.0), floor (cy - radius/10.0), floor (radius/6.0), 0, G_PI*2);
 			cairo_set_source_rgba (cr, highlight.r, highlight.g, highlight.b, 0.5);
 			cairo_fill (cr);
 		}
@@ -1539,10 +1532,10 @@ clearlooks_gummy_draw_checkbox (cairo_t                  *cr,
 	cairo_translate (cr, x, y);
 	cairo_set_line_width (cr, 1);
 
-	if (widget->xthickness > 2 && widget->ythickness > 2)
+	if (widget->xthickness >= 3 && widget->ythickness >= 3)
 	{
-		widget->style_functions->draw_inset (cr, &widget->parentbg, 0.5, 0.5,
-		                                     width-1, height-1, (widget->radius > 0)? 1 : 0, CR_CORNER_ALL);
+		widget->style_functions->draw_inset (cr, &widget->parentbg, 0, 0,
+		                                     width, height, (widget->radius > 0)? 1 : 0, CR_CORNER_ALL);
 
 		/* Draw the rectangle for the checkbox itself */
 		ge_cairo_rounded_rectangle (cr, 1.5, 1.5,
@@ -1591,6 +1584,26 @@ clearlooks_gummy_draw_checkbox (cairo_t                  *cr,
 	}
 }
 
+static void
+clearlooks_gummy_draw_icon_view_item (cairo_t                *cr,
+	                                  const ClearlooksColors *colors,
+	                                  const WidgetParameters *params,
+	                                  int x, int y, int width, int height)
+{
+	CairoColor color;
+	gfloat radius;
+
+	if (params->focus)
+		color = colors->base[params->state_type];
+	else
+		color = colors->base[GTK_STATE_ACTIVE];
+
+	radius = MIN (width / 2, params->radius);
+	radius = MIN (height / 2, radius);
+
+	clearlooks_draw_gummy_gradient (cr, x, y, width, height, &color, params->disabled, params->radius, CR_CORNER_ALL);
+}
+
 static void
 clearlooks_gummy_draw_focus (cairo_t                *cr,
                              const ClearlooksColors *colors,
@@ -1676,6 +1689,8 @@ clearlooks_gummy_draw_focus (cairo_t                *cr,
 			break;
 		case CL_FOCUS_SCALE:
 			break;
+		case CL_FOCUS_ICONVIEW:
+			break;
 		case CL_FOCUS_UNKNOWN:
 			/* Fallback to classic function, dots */
 			clearlooks_draw_focus (cr, colors, widget, focus, x, y, width, height);
@@ -1722,7 +1737,7 @@ clearlooks_gummy_draw_focus (cairo_t                *cr,
 }
 
 void
-clearlooks_register_style_gummy (ClearlooksStyleFunctions *functions)
+clearlooks_register_style_gummy (ClearlooksStyleFunctions *functions, ClearlooksStyleConstants *constants)
 {
 	functions->draw_button             = clearlooks_gummy_draw_button;
 	functions->draw_entry              = clearlooks_gummy_draw_entry;
@@ -1743,5 +1758,9 @@ clearlooks_register_style_gummy (ClearlooksStyleFunctions *functions)
 	functions->draw_statusbar          = clearlooks_gummy_draw_statusbar;
 	functions->draw_checkbox           = clearlooks_gummy_draw_checkbox;
 	functions->draw_radiobutton        = clearlooks_gummy_draw_radiobutton;
+	functions->draw_icon_view_item     = clearlooks_gummy_draw_icon_view_item;
 	functions->draw_focus              = clearlooks_gummy_draw_focus;
+
+	constants->topleft_highlight_shade = 1.3;
+	constants->topleft_highlight_alpha = 0.4;
 }
