@@ -1,22 +1,32 @@
-/**************************************************************************
-* GTK Industrial Theme
-* Designed by Tigert and Jimmac
-* Implemented by Christopher James Lahey
-*
-* Based on GTK Bluecurve Theme
-* Designed by Garrett LeSage
-* Implemented by Alexander Larsson
-*
-* Parts of the code based on Metal theme, Copyright:
-* Randy Gordon, randy@integrand.com
-* Owen Taylor <otaylor@redhat.com>
-*
-* Ported to cairo by:
-* Benjamin Berg <benjamin@sipsolutions.net>
-*
-* License: GPL (Gnu Public License)
-*
-**************************************************************************/
+/* Industrial - a cairo based GTK+ engine
+ * Copyright (C) 2001 Red Hat, Inc.
+ * Copyright (C) 2002 Ximian, Inc.
+ * Copyright (C) 2006-2007 Benjamin Berg <benjamin@sipsolutions.net>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
+ * 
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ * 
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ *
+ * Project contact: <gnome-themes-list@gnome.org>
+ *
+ *
+ * Written by Owen Taylor <otaylor@redhat.com>
+ * modified by Alexander Larsson <alexl@redhat.com>
+ * modified by Christopher James Lahey <clahey@ximian.com>
+ * modified by Benjamin Berg <benjamin@sipsolutions.net>
+ *
+ */
+
 
 #include "industrial_style.h"
 #include "industrial_rc_style.h"
@@ -26,6 +36,7 @@
 #include <ge-support.h>
 #include <cairo.h>
 #include <math.h>
+#include <stdio.h>
 
 #define LINE_OPACITY 0.4
 #define HANDLE_OPACITY 0.38
@@ -45,7 +56,7 @@
 
 #define IF_ROUNDED(style, rounded, otherwise) (GET_ROUNDED_BUTTONS (style) ? rounded : otherwise)
 
-static GtkStyleClass *parent_class;
+G_DEFINE_DYNAMIC_TYPE (IndustrialStyle, industrial_style, GTK_TYPE_STYLE)
 
 static void
 draw_rounded_gradient (cairo_t    *cr,
@@ -906,7 +917,7 @@ real_draw_box (GtkStyle      *style,
 			height -= 1;
 		}
 
-		parent_class->draw_flat_box (style, window, state_type,
+		GTK_STYLE_CLASS (industrial_style_parent_class)->draw_flat_box (style, window, state_type,
 					     shadow_type, area, widget, detail,
 					     x, y, width, height);
 	} else if (CHECK_DETAIL (detail, "slider")
@@ -928,11 +939,25 @@ real_draw_box (GtkStyle      *style,
 		else
 			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
 					   &bevel, NULL, CR_CORNER_ALL);
-	} else {
-		CairoColor bevel, bg;
+	} else if (CHECK_DETAIL (detail, "entry")) {
 		CairoCorners corners = CR_CORNER_ALL;
+		CairoColor bevel, bg, base;
 
-		if ((CHECK_DETAIL (detail, "entry") && ge_check_hint (GE_HINT_SPINBUTTON, GET_HINT (style), widget)) ||
+		ge_gdk_color_to_cairo (&style->bg[GTK_STATE_NORMAL], &bg);
+		ge_gdk_color_to_cairo (&style->base[state_type], &base);
+
+		/* Fill the background as it is initilized to base[NORMAL].
+		 * Relevant GTK+ bug: http://bugzilla.gnome.org/show_bug.cgi?id=513471
+		 * The fill only happens if no hint has been added by some application
+		 * that is faking GTK+ widgets. */
+		if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+		{
+			cairo_rectangle (cr, x, y, width, height);
+			ge_cairo_set_color (cr, &bg);
+			cairo_fill (cr);
+		}
+		
+		if (ge_check_hint (GE_HINT_SPINBUTTON, GET_HINT (style), widget) ||
 		    ge_check_hint (GE_HINT_COMBOBOX_ENTRY, GET_HINT (style), widget)) {
 			/* effectively cut one side off. */
 			width += 2;
@@ -940,6 +965,16 @@ real_draw_box (GtkStyle      *style,
 				x -= 2;
 		}
 		
+		
+		ge_gdk_color_to_cairo (&style->fg[state_type], &bevel);
+		bevel.a = GET_REAL_OPACITY (style, STANDARD_BORDER_OPACITY);
+
+		draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
+				   &bevel, &base, corners);
+	} else {
+		CairoColor bevel, bg;
+		CairoCorners corners = CR_CORNER_ALL;
+		
 		if (CHECK_DETAIL (detail, "spinbutton")) {
 			if (ge_widget_is_ltr (widget))
 				corners = CR_CORNER_TOPRIGHT | CR_CORNER_BOTTOMRIGHT;
@@ -957,7 +992,7 @@ real_draw_box (GtkStyle      *style,
 				draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
 						   &bevel, &bg, corners);
 			} else {
-				parent_class->draw_flat_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
+				GTK_STYLE_CLASS (industrial_style_parent_class)->draw_flat_box (style, window, state_type, shadow_type, area, widget, detail, x, y, width, height);
 			}
 		} else if (shadow_type != GTK_SHADOW_NONE) {
 			draw_rounded_rect (cr, x, y, width, height, IF_ROUNDED (style, 1.5, 0),
@@ -1039,7 +1074,7 @@ draw_focus (GtkStyle      *style,
 		return;
 	}
 	
-	parent_class->draw_focus (style, window, state_type, area, widget, detail, x, y, width, height);
+	GTK_STYLE_CLASS (industrial_style_parent_class)->draw_focus (style, window, state_type, area, widget, detail, x, y, width, height);
 }
 
 static void
@@ -1127,7 +1162,6 @@ real_draw_box_gap (GtkStyle       *style,
 	/* clip the gap area */
 	cairo_save (cr);
 	cairo_set_fill_rule (cr, CAIRO_FILL_RULE_EVEN_ODD);
-	cairo_rectangle (cr, x, y, width, height);
 
 	switch (gap_side) {
 	case GTK_POS_TOP:
@@ -1147,6 +1181,8 @@ real_draw_box_gap (GtkStyle       *style,
 	ge_cairo_set_color (cr, &bg);
 	if (fill)
 		cairo_fill_preserve (cr);
+
+	cairo_rectangle (cr, x, y, width, height);
 	cairo_clip (cr);
 
 	/* reset fill rule */
@@ -1632,4 +1668,71 @@ render_icon (GtkStyle            *style,
 	return stated;
 }
 
-#include "industrial_style_versioned_code.h"
+/* Non-drawing stuff */
+
+void
+industrial_style_register_types (GTypeModule *module)
+{
+	industrial_style_register_type (module);
+}
+
+static void
+industrial_style_copy (GtkStyle * style, GtkStyle * src)
+{
+	IndustrialStyle *industrial_style = INDUSTRIAL_STYLE (style);
+	IndustrialStyle *industrial_src = INDUSTRIAL_STYLE (src);
+
+	industrial_style->contrast = industrial_src->contrast;
+	industrial_style->rounded_buttons = industrial_src->rounded_buttons;
+
+	GTK_STYLE_CLASS (industrial_style_parent_class)->copy (style, src);
+}
+
+static void
+industrial_style_init_from_rc (GtkStyle * style, GtkRcStyle * rc_style)
+{
+	IndustrialStyle *industrial_style = INDUSTRIAL_STYLE (style);
+	GTK_STYLE_CLASS (industrial_style_parent_class)->init_from_rc (style, rc_style);
+
+	industrial_style->contrast = INDUSTRIAL_RC_STYLE (rc_style)->contrast;
+	industrial_style->rounded_buttons = INDUSTRIAL_RC_STYLE (rc_style)->rounded_buttons;
+	industrial_style->hint = INDUSTRIAL_RC_STYLE (rc_style)->hint;
+}
+
+static void
+industrial_style_init (IndustrialStyle * style)
+{
+	style->contrast = 1.0;
+	style->rounded_buttons = TRUE;
+	style->hint = 0;
+}
+
+static void
+industrial_style_class_init (IndustrialStyleClass * klass)
+{
+	GtkStyleClass *style_class = GTK_STYLE_CLASS (klass);
+
+	style_class->copy = industrial_style_copy;
+	style_class->init_from_rc = industrial_style_init_from_rc;
+
+	style_class->draw_focus = draw_focus;
+
+	style_class->draw_handle = draw_handle;
+	style_class->draw_vline = draw_vline;
+	style_class->draw_hline = draw_hline;
+	style_class->draw_slider = draw_slider;
+
+	style_class->draw_check = draw_check;
+	style_class->draw_box = draw_box;
+	style_class->draw_shadow = draw_shadow;
+	style_class->draw_box_gap = draw_box_gap;
+	style_class->draw_shadow_gap = draw_shadow_gap;
+	style_class->draw_extension = draw_extension;
+	style_class->draw_option = draw_option;
+	style_class->render_icon = render_icon;
+}
+
+static void
+industrial_style_class_finalize (IndustrialStyleClass *klass)
+{
+}
